@@ -1,4 +1,5 @@
-<?php $TRANSLATIONS = array(
+<?php
+$TRANSLATIONS = array(
 "Calendar" => "日曆",
 "Sunday" => "星期日",
 "Monday" => "星期一",
@@ -23,6 +24,8 @@
 "Birthday" => "生日",
 "Personal" => "個人",
 "Work" => "工作",
+"Daily" => "日",
+"Weekly" => "星期",
 "Date" => "日期",
 "Download" => "下載",
 "Edit" => "編輯",
@@ -32,3 +35,4 @@
 "Import" => "導入",
 "Unshare" => "取消分享"
 );
+$PLURAL_FORMS = "nplurals=1; plural=0;";
