@@ -4,7 +4,11 @@
 	<div id="header">
 		<a href="<?php print_unescaped(link_to('', 'index.php')); ?>"
 			title="" id="owncloud">
-			<div class="logo-wide svg"></div>
+			<div class="logo-wide svg">
+				<h1 class="hidden-visually">
+					<?php p($theme->getName()); ?>
+				</h1>
+			</div>
 		</a>
 		<div id="logo-claim" style="display:none;"><?php p($theme->getLogoClaim()); ?></div>
 		<div class="header-right">
@@ -23,7 +27,7 @@
 		</div>
 	</div>
 
-	<div id='gallery' class="hascontrols" data-token="<?php isset($_['token']) ? p($_['token']) : p(false) ?>"></div>
+	<div id='gallery' class="hascontrols" data-requesttoken="<?php p($_['requesttoken'])?>" data-token="<?php isset($_['token']) ? p($_['token']) : p(false) ?>"></div>
 </div>
 
 	<div class="push"></div><!-- for sticky footer -->
