@@ -74,9 +74,11 @@ class Cache {
 
 		if (!isset(self::$mimetypeIds[$mime])) {
 			try {
-				$result = \OC_DB::executeAudited('INSERT INTO `*PREFIX*mimetypes`(`mimetype`) VALUES(?)', array($mime));
-				self::$mimetypeIds[$mime] = \OC_DB::insertid('*PREFIX*mimetypes');
-				self::$mimetypes[self::$mimetypeIds[$mime]] = $mime;
+				$connection = \OC_DB::getConnection();
+				$connection->insertIfNotExist('*PREFIX*mimetypes', [
+					'mimetype'	=> $mime,
+				]);
+				$this->loadMimetypes();
 			} catch (\Doctrine\DBAL\DBALException $e) {
 				\OC_Log::write('core', 'Exception during mimetype insertion: ' . $e->getmessage(), \OC_Log::DEBUG);
 				return -1;
@@ -95,6 +97,8 @@ class Cache {
 	}
 
 	public function loadMimetypes() {
+		self::$mimetypeIds = self::$mimetypes = array();
+
 		$result = \OC_DB::executeAudited('SELECT `id`, `mimetype` FROM `*PREFIX*mimetypes`', array());
 		if ($result) {
 			while ($row = $result->fetchRow()) {
@@ -236,13 +240,19 @@ class Cache {
 			list($queryParts, $params) = $this->buildParts($data);
 			$queryParts[] = '`storage`';
 			$params[] = $this->getNumericStorageId();
-			$valuesPlaceholder = array_fill(0, count($queryParts), '?');
 
-			$sql = 'INSERT INTO `*PREFIX*filecache` (' . implode(', ', $queryParts) . ')'
-				. ' VALUES (' . implode(', ', $valuesPlaceholder) . ')';
-			\OC_DB::executeAudited($sql, $params);
+			$params = array_map(function($item) {
+				return trim($item, "`");
+			}, $params);
+			$queryParts = array_map(function($item) {
+				return trim($item, "`");
+			}, $queryParts);
+			$values = array_combine($queryParts, $params);
+			if (\OC::$server->getDatabaseConnection()->insertIfNotExist('*PREFIX*filecache', $values)) {
+				return (int)\OC_DB::insertid('*PREFIX*filecache');
+			}
 
-			return (int)\OC_DB::insertid('*PREFIX*filecache');
+			return $this->getId($file);
 		}
 	}
 
